@@ -1,4 +1,4 @@
-/** gaddr 1.0 - simple and small address book application using GTK+
+/** gaddr 1.1 - simple and small address book application using GTK+
     Copyright (C) 1999 Cory Lueninghoener (cluenin1@bigred.unl.edu)
 
     This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <gdk/gdk.h>
-#include "cardfile.xpm"
+#include "gaddr.xpm"
+
+#ifdef USE_GNOME
+#include <gnome.h>
+#endif
+
+#ifndef USE_GNOME
+#define PLAIN_GTK
+#endif
+
+#define VERSION "1.1.4"
 
 /*
   Initialize the pile o' widgets
@@ -50,7 +60,9 @@ GtkWidget *newdialoglabel;
 GtkWidget *newdialogbuttonok;
 GtkWidget *newdialogbuttoncancel;
 GtkWidget *listscroll;
+#ifdef PLAIN_GTK
 GtkWidget *menubar;
+#endif
 GtkWidget *filewindow;
 GtkWidget *confirmwindow;
 GtkWidget *confirmok;
@@ -59,6 +71,13 @@ GtkWidget *confirmlabel;
 GtkWidget *aboutwindow;
 GtkWidget *aboutlabel;
 GtkWidget *aboutbutton;
+GtkWidget *prefwindow;
+GtkWidget *autosavebutton;
+GtkWidget *prefframe;
+GtkWidget *prefvbox;
+GtkWidget *prefhbox;
+GtkWidget *prefok;
+GtkWidget *prefcancel;
 
 GdkPixmap* icon;
 
@@ -73,6 +92,7 @@ int selected_row, selected_column;
 int yes=1;
 int no=0;
 int edited=0;
+int autosave=0;
 
 void quit (GtkWidget*, GdkEvent*, gpointer);
 void new();
@@ -80,26 +100,68 @@ void save_as();
 void delete();
 void view(GtkWidget*, gpointer);
 void write_out();
+void read_rc();
 void newfile();
 void openfile();
 void delete_widget(GtkWidget*, gpointer);
 void about();
+void menuview(GtkWidget*);
+void menuedit(GtkWidget*);
+void newfileok();
+void openfileok();
+void prefs();
+static void drop_target(GtkWidget*, GdkDragContext*, gint, gint, GtkSelectionData*, guint, guint);
 
-/*
-  Callbacks for the view and edit buttons
-*/
-void menuview(GtkWidget *widget)
-{
-  view(widget, (gpointer) no);
-}
-void menuedit(GtkWidget *widget)
-{
-  view(widget, (gpointer) yes);
-}
+static GtkTargetEntry target_table[] = {
+  { "_NETSCAPE_URL", 0, 0 }
+};
 
-/*
-  Set up the menu
-*/
+#ifdef USE_GNOME
+GnomeUIInfo file_menu[] = {
+  GNOMEUIINFO_MENU_NEW_ITEM("New Addressbook","New Addressbook", newfile, NULL),
+  GNOMEUIINFO_MENU_OPEN_ITEM(openfile, NULL),
+  GNOMEUIINFO_MENU_SAVE_ITEM(write_out, NULL),
+  GNOMEUIINFO_MENU_SAVE_AS_ITEM(save_as, NULL),
+  GNOMEUIINFO_SEPARATOR,
+  GNOMEUIINFO_ITEM_STOCK("_Preferences", "Preferences", prefs, GNOME_STOCK_PIXMAP_PREFERENCES),
+  GNOMEUIINFO_SEPARATOR,
+  GNOMEUIINFO_MENU_EXIT_ITEM(quit, NULL),
+  GNOMEUIINFO_END
+};
+
+GnomeUIInfo card_menu[] = {
+  GNOMEUIINFO_ITEM_STOCK("_New Card", "New Card", new, GNOME_STOCK_MENU_NEW),
+  GNOMEUIINFO_ITEM_STOCK("View", "View Card", menuview, GNOME_STOCK_MENU_INDEX),
+  GNOMEUIINFO_ITEM_STOCK("Delete", "Delete Card", delete, GNOME_STOCK_MENU_TRASH),
+  GNOMEUIINFO_ITEM_STOCK("Edit", "Edit Card", menuedit, GNOME_STOCK_MENU_PROP),
+  GNOMEUIINFO_END
+};
+
+GnomeUIInfo help_menu[] = {
+  GNOMEUIINFO_HELP("gaddr"),
+  GNOMEUIINFO_MENU_ABOUT_ITEM(about, NULL),
+  GNOMEUIINFO_END
+};
+
+GnomeUIInfo menubar[] = {
+  GNOMEUIINFO_MENU_FILE_TREE(file_menu),
+  GNOMEUIINFO_SUBTREE("_Card", card_menu),
+  GNOMEUIINFO_MENU_HELP_TREE(help_menu),
+  GNOMEUIINFO_END
+};
+
+GnomeUIInfo toolbar[] = {
+  GNOMEUIINFO_ITEM_STOCK("View", NULL, menuview, GNOME_STOCK_MENU_INDEX),
+  GNOMEUIINFO_ITEM_STOCK("New", NULL, new, GNOME_STOCK_MENU_NEW),
+  GNOMEUIINFO_ITEM_STOCK("Edit", NULL, menuedit, GNOME_STOCK_MENU_PROP),
+  GNOMEUIINFO_ITEM_STOCK("Delete", NULL, delete, GNOME_STOCK_MENU_TRASH),
+  GNOMEUIINFO_ITEM_STOCK("Exit", NULL, quit, GNOME_STOCK_MENU_EXIT),
+  GNOMEUIINFO_END
+};
+
+#endif
+
+#ifdef PLAIN_GTK
 static GtkItemFactoryEntry menu_items[] = {
   {"/_File",         NULL,         NULL,        0, "<Branch>"},
   {"/File/_New",     NULL,         newfile,     0, NULL},
@@ -107,6 +169,8 @@ static GtkItemFactoryEntry menu_items[] = {
   {"/File/_Save",    "<control>S", write_out,   0, NULL},
   {"/File/Save _As", NULL,         save_as,     0, NULL},
   {"/File/sep1",     NULL,         NULL,        0, "<Separator>"},
+  {"/File/_Preferences", NULL,     prefs,       0, NULL},
+  {"/File/sep2",     NULL,         NULL,        0, "<Separator>"},
   {"/File/Quit",     "<control>Q", quit,        0, NULL},
 
   {"/_Card",         NULL,         NULL,        0, "<Branch>"},
@@ -119,9 +183,7 @@ static GtkItemFactoryEntry menu_items[] = {
   {"/_Help/About",   NULL,         about,       0, NULL},
 };
 
-/*
-  And create the main menu
-*/
+
 void get_main_menu(GtkWidget *window, GtkWidget ** menubar)
 {
   int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
@@ -136,16 +198,56 @@ void get_main_menu(GtkWidget *window, GtkWidget ** menubar)
   if (menubar)
     *menubar = gtk_item_factory_get_widget(item_factory, "<main>");
 }
+#endif
+
+/*
+  Callback for drag-and-drop operation
+*/
+
+static void drop_target(GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *data, guint info, guint time)
+{
+  gtk_text_insert( GTK_TEXT(widget), NULL, NULL, NULL, data->data, -1);
+}
+
+
+
+/*
+  Callbacks for the view and edit buttons
+*/
+void menuview(GtkWidget *widget)
+{
+  view(widget, (gpointer) no);
+}
+void menuedit(GtkWidget *widget)
+{
+  view(widget, (gpointer) yes);
+}
+
 
 /*
   Display the About box, so everybody can see my name.
 */
 void about()
 {
+#ifdef USE_GNOME
+  const gchar *authors[] = {
+    "Cory Lueninghoener <cluenin1@bigred.unl.edu>",
+    "Matthew Fisher <mfisher1@bigred.unl.edu>",
+    NULL
+  };
+  
+  aboutwindow =  gnome_about_new ( _("Gaddr"), 
+				   VERSION,
+				   _("(C) 1999 Cory Lueninghoener"),
+				   authors,
+				   "A GNOME aware version of Gaddr, the GTK Address Book.",
+				   "gaddr.xpm");
+#endif
+#ifdef PLAIN_GTK
   aboutwindow = gtk_window_new(GTK_WINDOW_DIALOG);
   gtk_container_border_width(GTK_CONTAINER(aboutwindow), 5);
   gtk_window_position (GTK_WINDOW(aboutwindow), GTK_WIN_POS_MOUSE);
-  gtk_window_set_title (GTK_WINDOW (aboutwindow), "About Gaddr 1.0.2");  
+  gtk_window_set_title (GTK_WINDOW (aboutwindow), "About Gaddr");  
   
   table = gtk_table_new(2, 3, FALSE);
   gtk_container_add(GTK_CONTAINER(aboutwindow), table);
@@ -163,9 +265,50 @@ void about()
   gtk_widget_show(aboutbutton);
   gtk_widget_show(aboutlabel);
   gtk_widget_show(table);
+#endif
   gtk_widget_show(aboutwindow);  
 }
 
+
+void read_rc()
+{
+  FILE *rcfile;
+  char inpref[255] = "";
+  char rc_file[255];
+  char *prefvalue;
+
+  sprintf(rc_file, "%s/.gaddrrc", getenv("HOME"));
+
+  if ((rcfile = fopen(rc_file, "r")) !=NULL)
+    {
+      while (fgets(inpref, 255, rcfile) !=NULL)
+	{
+	  inpref[strlen(inpref)-1] = 0;
+	  prefvalue = (strchr(inpref, ':')+1);
+	  if (strstr(inpref, "autosave"))
+	    {
+	      autosave = atoi(prefvalue);
+	    }
+	}
+    }
+}
+
+void write_rc()
+{
+  FILE *rcfile;
+  char rc_file[255];
+
+  sprintf(rc_file, "%s/.gaddrrc", getenv("HOME"));
+
+  if ((rcfile = fopen(rc_file, "w")) !=NULL)
+    {
+      fprintf(rcfile, "autosave:%d\n", autosave);
+      fclose(rcfile);
+    }
+}
+
+
+
 /*
   Read list of records from the selected file
 */
@@ -247,6 +390,49 @@ gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 /*
   Called when something is selected from the list
 */
+void set_selected(GtkWidget *widget, GdkEventButton *event)
+{
+  GtkWidget *rightmenu, *button;
+
+  gtk_clist_get_selection_info( GTK_CLIST(listbox), event->x, event->y, &selected_row, &selected_column);
+
+  if (event->type == GDK_2BUTTON_PRESS && event->button == 1)
+    { 
+      view(widget, (gpointer) no);
+    }
+  else if (event->type == GDK_BUTTON_PRESS && event->button == 3)
+    {
+      gtk_clist_select_row(GTK_CLIST(listbox), selected_row, selected_column);
+      
+      rightmenu = gtk_menu_new();
+
+      button = gtk_menu_item_new_with_label("Edit");
+      gtk_signal_connect(GTK_OBJECT(button), "activate",
+			 GTK_SIGNAL_FUNC(menuedit), NULL);
+      gtk_menu_append(GTK_MENU(rightmenu), button);
+      gtk_widget_show(button);
+
+      button = gtk_menu_item_new_with_label("View");
+      gtk_signal_connect(GTK_OBJECT(button), "activate",
+			 GTK_SIGNAL_FUNC(menuview), NULL);
+      gtk_menu_append(GTK_MENU(rightmenu), button);
+      gtk_widget_show(button);
+
+      button = gtk_menu_item_new_with_label("Delete");
+      gtk_signal_connect(GTK_OBJECT(button), "activate",
+			 GTK_SIGNAL_FUNC(delete), NULL);
+      gtk_menu_append(GTK_MENU(rightmenu), button);
+      gtk_widget_show(button);
+
+      gtk_menu_popup(GTK_MENU(rightmenu), NULL, NULL, NULL, NULL, event->button, event->time);
+
+    }
+  
+  return;
+}
+
+
+/*
 void set_selected(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
 {
   selected_row = row;
@@ -258,7 +444,7 @@ void set_selected(GtkWidget *widget, gint row, gint column, GdkEventButton *even
 
   return;
 }
-
+*/
 /*
   Allows you to view or edit the card
 */
@@ -285,6 +471,16 @@ void view(GtkWidget *widget, gpointer   data)
   gtk_window_set_title(GTK_WINDOW(viewwindow), title);
 
   textbox = gtk_text_new(NULL, NULL);
+  if (data != 0) {
+    gtk_drag_dest_set (textbox,
+		       GTK_DEST_DEFAULT_ALL,
+		       target_table, 1,
+		       GDK_ACTION_COPY);
+    gtk_signal_connect (GTK_OBJECT (textbox), "drag_data_received",
+			GTK_SIGNAL_FUNC (drop_target),
+			textbox);
+  }
+  
   table = gtk_table_new(3, 1, FALSE);
   closebutton = gtk_button_new_with_label("Close");
 
@@ -342,7 +538,7 @@ void newok(GtkWidget *widget)
   
   while ((gtk_clist_get_text(GTK_CLIST(listbox), counter, 0, &namechar)!=0)&&(compare > 0))
     {
-      compare = strcmp(entryname[0][0], namechar);
+      compare = strcoll(entryname[0][0], namechar);
       if (compare > 0)
 	counter++;
     }
@@ -350,14 +546,82 @@ void newok(GtkWidget *widget)
   add_list(addressdb, entryname[0][0], "");
   gtk_widget_destroy(newdialogwindow);
   gtk_clist_select_row(GTK_CLIST(listbox), counter, 0);
+  selected_row = counter;
   view(widget, (gpointer) yes);
 }
 
+
+/* Callbacks for the Gnome versions of the dialog boxes */
+#ifdef USE_GNOME
+void new_clicked(GnomeDialog *dialog, int button, gpointer data)
+{
+  switch(button) {
+  case 0:
+    newok(GTK_WIDGET(dialog));
+    return;
+  case 1:
+    gtk_widget_destroy(GTK_WIDGET(dialog));
+    return;
+  }
+}
+
+void quit_clicked(GnomeDialog *dialog, int button, gpointer data)
+{
+  switch(button) {
+  case 0:
+    gtk_main_quit();
+    return;
+  case 1:
+    return;
+  }
+}
+
+void openfile_clicked(GnomeDialog *dialog, int button, gpointer data)
+{
+  switch(button) {
+  case 0:
+    openfileok();
+    return;
+  case 1:
+    return;
+  }
+}
+
+void newfile_clicked(GnomeDialog *dialog, int button, gpointer data)
+{
+  switch(button) {
+  case 0:
+    newfileok();
+    return;
+  case 1:
+    return;
+  }
+}
+
+
+#endif
+
 /*
   Wanna create a new card?  This does it...
 */
 void new()
 {
+#ifdef USE_GNOME
+  newdialogwindow = gnome_dialog_new("Create New Card", GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
+  newdialoglabel = gtk_label_new("Enter Name:");
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(newdialogwindow)->vbox), newdialoglabel, TRUE, TRUE, 0);
+  gtk_widget_show(newdialoglabel);
+  newdialoginput = gtk_entry_new_with_max_length(39);
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(newdialogwindow)->vbox), newdialoginput, TRUE, TRUE, 0);
+  gtk_widget_show(newdialoginput);
+
+  gtk_signal_connect(GTK_OBJECT(newdialogwindow),"clicked", GTK_SIGNAL_FUNC(new_clicked), NULL);
+
+  gtk_widget_show(newdialogwindow);
+#endif
+
+/* Original Gaddr */
+#ifdef PLAIN_GTK
   newdialogwindow = gtk_window_new(GTK_WINDOW_DIALOG);
   gtk_container_border_width(GTK_CONTAINER(newdialogwindow), 5);
   gtk_window_position (GTK_WINDOW(newdialogwindow), GTK_WIN_POS_MOUSE);
@@ -396,49 +660,66 @@ void new()
   gtk_widget_show(newdialoglabel);
   gtk_widget_show(table);
   gtk_widget_show(newdialogwindow);
+#endif
 }
 
 
 /* 
    This callback quits the program 
 */
-void quit (GtkWidget *widget, GdkEvent *event, gpointer data)
+
+void quit(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
-  if (edited == 1)
-    {
-      confirmwindow = gtk_window_new(GTK_WINDOW_DIALOG);
-      gtk_container_border_width(GTK_CONTAINER(confirmwindow), 5);
-      gtk_window_position (GTK_WINDOW(confirmwindow), GTK_WIN_POS_MOUSE);
-      gtk_window_set_title (GTK_WINDOW (confirmwindow), "Really quit?");
-      
-      
-      table = gtk_table_new(3, 2, TRUE);
-      gtk_container_add(GTK_CONTAINER(confirmwindow), table);
-      
-      confirmlabel = gtk_label_new("File is unsaved.  Are you\nsure you want to quit?");
-      gtk_table_attach_defaults(GTK_TABLE(table), confirmlabel, 0, 2, 0, 2);
-      
-      confirmok = gtk_button_new_with_label("Ok");
-      gtk_table_attach_defaults(GTK_TABLE(table), confirmok, 0, 1, 2, 3);
-      
-      gtk_signal_connect(GTK_OBJECT(confirmok), "clicked",
-      			 GTK_SIGNAL_FUNC(gtk_main_quit),
-			 NULL);
-      
-      confirmcancel = gtk_button_new_with_label("Cancel");
-      gtk_table_attach_defaults(GTK_TABLE(table), confirmcancel, 1, 2, 2, 3);
-      
-      gtk_signal_connect(GTK_OBJECT(confirmcancel), "clicked",
-			 GTK_SIGNAL_FUNC(delete_widget),
-			 confirmwindow);
-      
-      gtk_widget_show(confirmok);
-      gtk_widget_show(confirmcancel);
-      gtk_widget_show(confirmlabel);
-      gtk_widget_show(table);
-      gtk_widget_show(confirmwindow);
-      
-    }
+  if (autosave == 1) {
+    write_out();
+    gtk_main_quit();
+  }
+  
+
+  if (edited == 1) {
+/* Gnome Version */
+#ifdef USE_GNOME
+    confirmwindow = gnome_message_box_new("File is unsaved.  Are you\nsure you want to quit?", GNOME_MESSAGE_BOX_WARNING, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
+    gtk_signal_connect(GTK_OBJECT(confirmwindow),"clicked", GTK_SIGNAL_FUNC(quit_clicked), NULL);
+    gtk_widget_show(confirmwindow);
+#endif
+
+/* Original gaddr */
+#ifdef PLAIN_GTK
+
+    confirmwindow = gtk_window_new(GTK_WINDOW_DIALOG);
+    gtk_container_border_width(GTK_CONTAINER(confirmwindow), 5);
+    gtk_window_position (GTK_WINDOW(confirmwindow), GTK_WIN_POS_MOUSE);
+    gtk_window_set_title (GTK_WINDOW (confirmwindow), "Really quit?");
+    
+    
+    table = gtk_table_new(3, 2, TRUE);
+    gtk_container_add(GTK_CONTAINER(confirmwindow), table);
+    
+    confirmlabel = gtk_label_new("File is unsaved.  Are you\nsure you want to quit?");
+    gtk_table_attach_defaults(GTK_TABLE(table), confirmlabel, 0, 2, 0, 2);
+    
+    confirmok = gtk_button_new_with_label("Ok");
+    gtk_table_attach_defaults(GTK_TABLE(table), confirmok, 0, 1, 2, 3);
+    
+    gtk_signal_connect(GTK_OBJECT(confirmok), "clicked",
+		       GTK_SIGNAL_FUNC(gtk_main_quit),
+		       NULL);
+    
+    confirmcancel = gtk_button_new_with_label("Cancel");
+    gtk_table_attach_defaults(GTK_TABLE(table), confirmcancel, 1, 2, 2, 3);
+    
+    gtk_signal_connect(GTK_OBJECT(confirmcancel), "clicked",
+		       GTK_SIGNAL_FUNC(delete_widget),
+		       confirmwindow);
+    
+    gtk_widget_show(confirmok);
+    gtk_widget_show(confirmcancel);
+    gtk_widget_show(confirmlabel);
+    gtk_widget_show(table);
+    gtk_widget_show(confirmwindow);
+#endif    
+  }
  
   else
     gtk_main_quit ();
@@ -550,6 +831,13 @@ void openfile()
 {
   if (edited == 1)
     {
+#ifdef USE_GNOME
+      confirmwindow = gnome_message_box_new("File is unsaved.  Are you\nsure you want to open a new one?", GNOME_MESSAGE_BOX_WARNING, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
+      gtk_signal_connect(GTK_OBJECT(confirmwindow),"clicked", GTK_SIGNAL_FUNC(openfile_clicked), NULL);
+      gtk_widget_show(confirmwindow);
+#endif
+
+#ifdef PLAIN_GTK
       confirmwindow = gtk_window_new(GTK_WINDOW_DIALOG);
       gtk_container_border_width(GTK_CONTAINER(confirmwindow), 5);
       gtk_window_position (GTK_WINDOW(confirmwindow), GTK_WIN_POS_MOUSE);
@@ -581,6 +869,7 @@ void openfile()
       gtk_widget_show(confirmlabel);
       gtk_widget_show(table);
       gtk_widget_show(confirmwindow);
+#endif
     }
   else
     openfileok();
@@ -595,101 +884,218 @@ void newfileok()
   wipe_list(addressdb);
   addressdb=init_list();
   edited = 0;
+/*
   if (confirmwindow != NULL)
     {
       gtk_widget_destroy(GTK_WIDGET(confirmwindow));
       confirmwindow = NULL;
     }
+*/
   gtk_clist_clear(GTK_CLIST(listbox));
 }
 
+
+
 /*
   Brings up the confirmation to create a new file if needed
 */
+
 void newfile()
 {
-  if (edited == 1)
-    {
-      confirmwindow = gtk_window_new(GTK_WINDOW_DIALOG);
-      gtk_container_border_width(GTK_CONTAINER(confirmwindow), 5);
-      gtk_window_position (GTK_WINDOW(confirmwindow), GTK_WIN_POS_MOUSE);
-      gtk_window_set_title (GTK_WINDOW (confirmwindow), "File Unsaved!");
-      
-      
-      table = gtk_table_new(3, 2, TRUE);
-      gtk_container_add(GTK_CONTAINER(confirmwindow), table);
-      
-      confirmlabel = gtk_label_new("File is unsaved.  Are you\nsure you want to create a new one?");
-      gtk_table_attach_defaults(GTK_TABLE(table), confirmlabel, 0, 2, 0, 2);
-      
-      confirmok = gtk_button_new_with_label("Ok");
-      gtk_table_attach_defaults(GTK_TABLE(table), confirmok, 0, 1, 2, 3);
-      
-      gtk_signal_connect(GTK_OBJECT(confirmok), "clicked",
-      			 GTK_SIGNAL_FUNC(newfileok),
-			 NULL);
-      
-      confirmcancel = gtk_button_new_with_label("Cancel");
-      gtk_table_attach_defaults(GTK_TABLE(table), confirmcancel, 1, 2, 2, 3);
-      
-      gtk_signal_connect(GTK_OBJECT(confirmcancel), "clicked",
-			 GTK_SIGNAL_FUNC(delete_widget),
-			 confirmwindow);
-      
-      gtk_widget_show(confirmok);
-      gtk_widget_show(confirmcancel);
-      gtk_widget_show(confirmlabel);
-      gtk_widget_show(table);
-      gtk_widget_show(confirmwindow);
-    }
+  if (edited == 1) {
+#ifdef USE_GNOME
+    confirmwindow = gnome_message_box_new("File is unsaved.  Are you\nsure you want to create a new one?", GNOME_MESSAGE_BOX_WARNING, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
+    gtk_signal_connect(GTK_OBJECT(confirmwindow),"clicked", GTK_SIGNAL_FUNC(newfile_clicked), NULL);
+    gtk_widget_show(confirmwindow);
+#endif
+
+/* Original Gaddr*/
+#ifdef PLAIN_GTK
+    confirmwindow = gtk_window_new(GTK_WINDOW_DIALOG);
+    gtk_container_border_width(GTK_CONTAINER(confirmwindow), 5);
+    gtk_window_position (GTK_WINDOW(confirmwindow), GTK_WIN_POS_MOUSE);
+    gtk_window_set_title (GTK_WINDOW (confirmwindow), "File Unsaved!");
+    
+    
+    table = gtk_table_new(3, 2, TRUE);
+    gtk_container_add(GTK_CONTAINER(confirmwindow), table);
+    
+    confirmlabel = gtk_label_new("File is unsaved.  Are you\nsure you want to create a new one?");
+    gtk_table_attach_defaults(GTK_TABLE(table), confirmlabel, 0, 2, 0, 2);
+    
+    confirmok = gtk_button_new_with_label("Ok");
+    gtk_table_attach_defaults(GTK_TABLE(table), confirmok, 0, 1, 2, 3);
+    
+    gtk_signal_connect(GTK_OBJECT(confirmok), "clicked",
+		       GTK_SIGNAL_FUNC(newfileok),
+		       NULL);
+    
+    confirmcancel = gtk_button_new_with_label("Cancel");
+    gtk_table_attach_defaults(GTK_TABLE(table), confirmcancel, 1, 2, 2, 3);
+    
+    gtk_signal_connect(GTK_OBJECT(confirmcancel), "clicked",
+		       GTK_SIGNAL_FUNC(delete_widget),
+		       confirmwindow);
+    
+    gtk_widget_show(confirmok);
+    gtk_widget_show(confirmcancel);
+    gtk_widget_show(confirmlabel);
+    gtk_widget_show(table);
+    gtk_widget_show(confirmwindow);
+#endif
+  }
   else
     newfileok();
 }
 
+
+void prefokcb()
+{
+  if (GTK_TOGGLE_BUTTON (autosavebutton)->active)
+    {
+      autosave = 1;
+    }
+  if (!GTK_TOGGLE_BUTTON(autosavebutton)->active)
+    {
+      autosave = 0;
+    }
+  gtk_widget_destroy(prefwindow);
+  prefwindow = NULL;
+  write_rc();
+}
+
+void prefcancelcb()
+{
+  gtk_widget_destroy(prefwindow);
+  prefwindow = NULL;
+}
+
+
+#ifdef USE_GNOME
+void pref_clicked(GnomeDialog *dialog, int button, gpointer data)
+{
+  switch(button) {
+  case 0:
+    prefokcb();
+    return;
+  case 1:
+    prefcancelcb();
+    return;
+  case 2:
+    prefcancelcb();
+    return;
+  }
+}
+#endif
+
+void prefs() 
+{
+  
+  if (prefwindow != NULL)
+    {
+      return;
+    }
+  gtk_clist_set_selection_mode(GTK_CLIST(listbox), GTK_SELECTION_EXTENDED);
+
+#ifdef USE_GNOME
+  prefwindow = gnome_dialog_new("Preferences", GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CLOSE, NULL);
+  
+    prefframe = gtk_frame_new("General Preferences");
+    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(prefwindow)->vbox), prefframe, FALSE, FALSE, 2);
+    gtk_widget_show(prefframe);
+
+    gtk_signal_connect(GTK_OBJECT(prefwindow),"clicked", GTK_SIGNAL_FUNC(pref_clicked), NULL);
+#endif
+
+#ifdef PLAIN_GTK
+  prefwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title(GTK_WINDOW(prefwindow), "Preferences");
+
+  prefvbox = gtk_vbox_new(FALSE, 2);
+  gtk_container_add(GTK_CONTAINER(prefwindow), prefvbox);
+  gtk_widget_show(prefvbox);
+
+  prefframe = gtk_frame_new("General Preferences");
+  gtk_box_pack_start(GTK_BOX(prefvbox), prefframe, FALSE, FALSE, 2);
+  gtk_widget_show(prefframe);
+
+  prefhbox = gtk_hbox_new(TRUE, 2);
+  gtk_box_pack_start(GTK_BOX(prefvbox), prefhbox, TRUE, TRUE, 2);
+  gtk_widget_show(prefhbox);
+  
+  prefok = gtk_button_new_with_label("OK");
+  gtk_box_pack_start(GTK_BOX(prefhbox), prefok, TRUE, TRUE, 2);
+  gtk_signal_connect(GTK_OBJECT(prefok),"clicked", GTK_SIGNAL_FUNC(prefokcb), prefwindow);
+  gtk_widget_show(prefok);
+  
+  prefcancel = gtk_button_new_with_label("Cancel");
+  gtk_box_pack_start(GTK_BOX(prefhbox), prefcancel, TRUE, TRUE, 2);
+  gtk_signal_connect(GTK_OBJECT(prefcancel),"clicked", GTK_SIGNAL_FUNC(prefcancelcb), prefwindow);
+  gtk_widget_show(prefcancel);
+#endif
+
+  prefvbox = gtk_vbox_new(FALSE, 2);
+  gtk_container_add(GTK_CONTAINER(prefframe), prefvbox);
+  gtk_widget_show(prefvbox);
+
+  autosavebutton = gtk_check_button_new_with_label("Autosave on Exit");
+  gtk_box_pack_start(GTK_BOX(prefvbox), autosavebutton, FALSE, FALSE, 2);
+  if (autosave == 1)
+    {
+      GTK_TOGGLE_BUTTON(autosavebutton)->active = TRUE;
+    }
+  gtk_widget_show(autosavebutton);
+
+  gtk_widget_show(prefwindow);
+  
+  return;
+}
+
+
+
 /*
   The main function, of course
 */
 int main (int argc, char *argv[])
 {
-  gtk_init (&argc, &argv);
-  addressdb=init_list();
+#ifdef USE_GNOME
+  gnome_init ("gaddr", VERSION, argc, argv);
+  window = gnome_app_new("Gaddr", "Gaddr");
+#endif
 
-  //strcpy(book_file, "gaddrbook");
+#ifdef PLAIN_GTK
+  gtk_set_locale();
+  gtk_init (&argc, &argv);
+  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title (GTK_WINDOW (window), "Gaddr");
+#endif
+
+  addressdb=init_list();
 
   sprintf(book_file, "%s/.gaddrbook", getenv("HOME"));
 
-  /* create a new window */
-  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   iconwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  
-  /* set the window title */
-  gtk_window_set_title (GTK_WINDOW (window), "Gaddr");
 
-  //  icon=gdk_pixmap_create_from_xpm(window->window, NULL, NULL, "cardfile.xpm");
-  //  gdk_window_set_icon(window->window, NULL, icon, NULL);
-  
-  /* set a handler for delete_event that immediately
-   * exits GTK. */
   gtk_signal_connect (GTK_OBJECT (window), "delete_event",
 		      GTK_SIGNAL_FUNC (quit), NULL);
   
-  /* sets the border width of the window. */
   gtk_container_border_width (GTK_CONTAINER(window), 0);
-  
-  /* create a 2x2 table */
   table = gtk_table_new (8, 3, FALSE);
   
   /* put the table in the main window */
-  gtk_container_add (GTK_CONTAINER (window), table);
-  
-  //------------------------------------------------------------
 
+#ifdef USE_GNOME
+  gnome_app_set_contents (GNOME_APP (window), table);
+  gnome_app_create_menus (GNOME_APP (window), menubar);
+  gnome_app_create_toolbar (GNOME_APP (window), toolbar);
+#endif
+  
+#ifdef PLAIN_GTK
+  gtk_container_add (GTK_CONTAINER (window), table);
   get_main_menu(window, &menubar);
   gtk_table_attach_defaults(GTK_TABLE(table), menubar, 0, 3, 0, 1);
   gtk_widget_show(menubar);
-  
-  //--------------------------------------------------------------
-
+#endif
 
   /*_____List Box_____*/
 
@@ -700,13 +1106,24 @@ int main (int argc, char *argv[])
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (listscroll),
 				  GTK_POLICY_NEVER,
 				  GTK_POLICY_AUTOMATIC);
-  gtk_widget_set_usize(listscroll, 260, 200);
-
+/*
   gtk_signal_connect(GTK_OBJECT(listbox), "select_row",
 		     GTK_SIGNAL_FUNC(set_selected),
 		     NULL);
+*/
+  gtk_signal_connect(GTK_OBJECT(listbox), "button_press_event",
+		     GTK_SIGNAL_FUNC(set_selected),
+		     NULL);
   gtk_clist_set_selection_mode(GTK_CLIST(listbox), GTK_SELECTION_SINGLE);
 
+#ifdef USE_GNOME
+  gtk_widget_set_usize(listscroll, 265, 180);
+#endif
+#ifdef PLAIN_GTK
+  gtk_widget_set_usize(listscroll, 260, 200);
+#endif
+
+#ifdef PLAIN_GTK
   /*_____Vertical Seperator_____*/
 
   vsep1 = gtk_vseparator_new();
@@ -780,16 +1197,19 @@ int main (int argc, char *argv[])
   gtk_widget_show(hsep2);
   gtk_widget_show(vsep1);
 
+#endif
+
   gtk_widget_show (table);
-  gtk_widget_show (window);
+  gtk_widget_show_all (window);
   
-  icon=gdk_pixmap_create_from_xpm_d(window->window, NULL, NULL, cardfile_xpm);
+  icon=gdk_pixmap_create_from_xpm_d(window->window, NULL, NULL, gaddr_xpm);
 
   gdk_window_set_icon(window->window, NULL, icon, NULL);
 
   if (argv[1] != NULL)
     strcpy(book_file, argv[1]);
-
+  
+  read_rc();
   read_in();
   
   gtk_main ();
